# Usage example:
#
#   nmake -f ../CodeGenerator/Scripts/CodeGenerator.mak \
#   COMPANY=DE_EPAGES CARTRIDGE=Object MODELS=Models/Class.pdm \
#   CVS_PATH=$(CVS_PATH) PERL=$(PERL) generate
#
# Targets
#
# generate    Generates code into the Generated directory.
#         No CVS actions.
#         Use this target to test new code, if you dont have cvs.
#
# generate_update   Generates code into the Generated directory and unmark changed files via cvs update.
#         Use this target to test new code, if you have cvs.
#         This is the default target.
#
# generate_merge    Generates code into the Generated directory.
#         Commits changes to the "B_$(CVS_BRANCH)_gen" branch in CVS, but no new files.
#         Merges all changes between the top of the "B_$(CVS_BRANCH)_gen" branch and
#         the "M_$(CVS_BRANCH)_gen" label into the working directory.
#         Tags the "B_$(CVS_BRANCH)_gen" branch with the label "M_$(CVS_BRANCH)_gen".
#         Use this target also to add new files, for example to apply updated code templates.
#
# Variables
#
# COMPANY     Company directory name, default: DE_EPAGES
# CARTRIDGE   Cartridge directory name
# MODELS      Model file name
# CVS_BRANCH    development branch (main for main-branch)
#       Example: CVS_BRANCH=f_user_feature
#         Development branch is     B_f_user_feature
#         CodeGenerator branch is   B_f_user_feature_gen
#         CodeGenerator merge tag is  M_f_user_feature_gen
#       Example: CVS_BRANCH=main
#         Development branch is     (main no branch)
#         CodeGenerator branch is   generated
#         CodeGenerator merge tag is  merged
#
# CVS_STARTBRANCH release/patch branch (main for main-branch)

COMPANY = DE_EPAGES
CARTRIDGE =
MODELS =
XML_MODELS2 = $(MODELS:.pdm=.xml)
XML_MODELS = $(XML_MODELS2:.zargo=.xml)
DE_EPAGES = ../../DE_EPAGES
CVS_BRANCH = main
CVS_STARTBRANCH = main

PERL = perl
CVS = cvs
MKDIR = mkdir
DEL = $(PERL) -MExtUtils::Command -e rm_f

BUILDFILE = $(DE_EPAGES)/CodeGenerator/Templates/epages/BuildCentralDAL.xml
GENERATE = $(PERL) $(DE_EPAGES)/CodeGenerator/Scripts/CreateCode2.pl -buildfile $(BUILDFILE) -targetdir Generated -company $(COMPANY) -cartridge $(CARTRIDGE)
COMPANY_CARTRIDGE = COMPANY=$(COMPANY) CARTRIDGE=$(CARTRIDGE) DE_EPAGES="$(DE_EPAGES)"

MAKE_OS_INDEPENDEND = $(MAKE) -f $(DE_EPAGES)/CodeGenerator/Scripts/makefile $(COMPANY_CARTRIDGE) CVS=$(CVS) CVS_BRANCH=$(CVS_BRANCH) CVS_PATH=$(CVS_PATH)

generate_update: get_gen generate update_gen clean
generate_merge:  get_gen generate update_gen commit_gen add_gen merge tag_merged clean
generate_new:  generate_merge 
resolve_conficts: update tag_merged $(DE_EPAGES)/CodeGenerator/Scripts

Init:
  (cd $(DE_EPAGES)/CodeGenerator/Scripts && $(PERL) Makefile.PL)
  
Generated: Init
  $(MKDIR) Generated
  (cd Generated && $(MKDIR) $(COMPANY)) 
  $(MAKE_OS_INDEPENDEND) gen

.SUFFIXES: .xml .zargo .pdm .xmi

.zargo.xmi:
  unzip -o -d Models $< $(@F)
  
XMIFILES = $(XML_MODELS:.xml=.xmi)

$(XMIFILES):

.xmi.xml:
  $(PERL) $(DE_EPAGES)/CodeGenerator/Scripts/CreateModelFromUML.pl -targetdir Generated -cartridge $(CARTRIDGE) -company $(COMPANY) $<
  -(cd Generated/$(COMPANY)/$(CARTRIDGE) && cvs add $@)
  (cd Generated/$(COMPANY)/$(CARTRIDGE) && cvs commit -m "new generated model" $@)
  cvs -q update -j merged -j generated $@
  cvs -q tag merged $@
  
.pdm.xml:
  $(PERL) $(DE_EPAGES)/CodeGenerator/Scripts/CreateCode.pl -buildfile $(DE_EPAGES)/CodeGenerator/Templates/epages/BuildNewModel.xml -targetdir Generated -company $(COMPANY) -cartridge $(CARTRIDGE)  $<
  -(cd Generated/$(COMPANY)/$(CARTRIDGE) && cvs add $@)
  (cd Generated/$(COMPANY)/$(CARTRIDGE) && cvs commit -m "new generated model" $@)
  cvs -q update -j merged -j generated $@
  cvs -q tag merged $@
  
generate: Init
  $(GENERATE) $(XML_MODELS)

get_gen: Generated
  $(MAKE_OS_INDEPENDEND) $@

update_gen:
  (cd Generated && cd $(COMPANY) && cd $(CARTRIDGE) && $(CVS) -q update -d)

commit_gen:
  (cd Generated && cd $(COMPANY) && cd $(CARTRIDGE) && $(CVS) -q commit -m "code generator")

add_gen:
  $(MAKE_OS_INDEPENDEND) $@

merge:
  $(CVS) -q update -d -I Generated
  $(MAKE_OS_INDEPENDEND) $@

update:
  $(CVS) -q update -d -I Generated

tag_merged:
  $(MAKE_OS_INDEPENDEND) $@

create_development_branches:
  $(MAKE_OS_INDEPENDEND) $@

merge_development_branches:
  $(MAKE_OS_INDEPENDEND) $@
  
clean:    
  -$(DEL) $(DE_EPAGES)/CodeGenerator/Scripts/makefile
