/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ui.linkpicker
 *
 * @revision		$Revision: 1.1.2.3 $
 */

jQuery.plugin( 'de_epages.presentation.ui.linkpicker', {
	plugin:	[
		'jQuery.ui.widget',
		'jQuery.tmpl',
		'jQuery.metaparse',
		'ep.dict',
		'ep.core',
		'ep.ui.dialog',
		'ep.ajax',
		'ep.ui.input',
		'jQuery.jstree',
		'jQuery.jstree.themes',
		'jQuery.jstree.json_data',
		'de_epages.core'
		],
	tmpl:	'de_epages.presentation.ui.linkpicker',
	dict:	'de_epages.presentation.dictionary'
}, function( $, undefined ){

/*
 * @dictionary		ep.dict
 *
 * @translation		{EditLink}
 *                {Apply}
 *								{LinkText}
 *								{Link}
 *								{LinkToWebsite}
 *								{OpenInNewWindow}
 *								{MoreLinks}
 *								{Choice}
 *								{Anchor}
 *								{Example}
 *								{ExplainExternalLinks}
 *								{ExampleExternalLink}
 */


	// widget to display history
	$.widget( 'de_epages.presentation.ui.Linkpicker', {

		dict: ep.dict('ep.dict'),
		options: {
			startpageId: ep.config.startpageId,
			callback: $.noop,
			autoOpen: false,
			title: '',
			url: '',
			useAbsolute: false,
			useSSL: false,
			anchors : undefined,
			newWindow: true,
			modal: false
		},

		_create: function(){
			var self = this,
				o = self.options;

			// load template and push named tmpl elements to current instance
			$.extend(
				self,
				$.tmpl( 'de_epages.presentation.ui.linkpicker', [{title: o.title, url: o.url, newWindow: o.newWindow}] )
				 .dictParse('ep.dict',true)
				 .appendTo('body')
				 .hide()
				 .tmplItem('elements')
			);
			self.dialog
				.find('.ep-js')
				.metaparse();

			self.previewLink.attr('title',self.dict.translate(self.previewLink.attr('title')));

//#JSCOVERAGE_IF false
			//bind preview link
			self.previewLink.on('click',function(){
				var currentUrl = self.urlInput.val();

				if(currentUrl != ''){
					if(currentUrl.indexOf("?") != 0) {
						if(currentUrl.split('//').length == 1){
							currentUrl = "http://" + currentUrl;
						}
					} else{
						currentUrl = ep.config.storeFrontUrl + currentUrl;
					}
					window.open(currentUrl,'LinkpickerPreview');
				}
			});
//#JSCOVERAGE_ENDIF

			self.anchorSelect.on('click',$.proxy(self,'_onSelectChange'));
		},

		open: function(data){
			var self = this,
				o = self.options;

			$.extend( o,data);

			if(self.dialog.data('epUiDialog')){
				self.dialog.uiDialog('open');
			}
			else{
				self.dialog.uiDialog({
					modal: o.modal,
					width: 450,
					title: self.dict.translate('EditLink'),
					buttons: [
						{
							text: self.dict.translate('Apply'),
							click: function(){
								ep(this).uiDialog('close');
								if(o.callback){
									var titleVal = self.titleInput.val(),
										urlVal = self.urlInput.val();

									o.callback(urlVal,titleVal ? titleVal : urlVal, o.newWindow ? self.newWindowSelect.is(':checked') : false);
								}
							}
						}
					]
				});

				self._createTree();
				self._getAdvancedLinks();
			}

			if(o.url){
				self.urlInput.val(data.url);
			}
			else {
				self.urlInput.val('');
			}

			if(o.title){
				self.titleInput
					.val(data.title)
					.attr('disabled', false);
			}
			else {
				self.titleInput
					.val('')
					.attr('disabled',false);
			}

			if(o.anchors){
				self.anchorSelect.parent().show();
				self.anchorSelect.children('option').remove();
				$(o.anchors).each(function(i,elem){
					$('<option>')
						.val('#' + elem)
						.html(elem)
						.appendTo(self.anchorSelect);
					});
			}
			else{
				self.anchorSelect.parent().hide();
			}
		},

		_createTree: function(){
			var self = this;

			self.tree.jstree({
					//config for tree themes plugin
					themes : {
						theme : 'bo',
						dots  : true,
						icons : true
					},
					//config for tree json_data plugin
					json_data : {
						ajax : {
							url : '?',
							type: 'POST',
							dataType: 'json',
							beforeSend: function( xhr, s ){
								xhr.setRequestHeader('Accept', 'text/x-json');
							},
							context: this,
							data: $.proxy(self,'_generateTreeJSONData')
						}
					},
					plugins : ['themes', 'json_data']
				})
				.on('click','a', $.proxy(self,'_onClickTreeNode'));
		},

		_onClickTreeNode: function(evt){
			evt.preventDefault();
			evt.stopPropagation();

			var self = this,
				o = self.options,
				leafNode = $(evt.currentTarget).parent(),
				leafNodeData = leafNode.data('jstree'),
				hrefValue = o.useSSL && leafNodeData.WebUrlSSL ? leafNodeData.WebUrlSSL : leafNodeData.WebUrl;

			if(o.useAbsolute == false){
				var found = (/(\?.*)$/).exec( hrefValue );
				if(found && found[1]){
					hrefValue = found[1];
				}
			}

			self.urlInput.val(hrefValue);

			if(!self.titleInput.attr('disabled')){
				self.titleInput.val(self.tree.jstree('get_text',leafNode));
			}
		},


		_generateTreeJSONData: function(node){
			var returnData = {
				ViewAction : 'JSONChildrenJSTree'
			};

			if(node == -1){
				returnData.WithParent = 1;
				returnData.ObjectID = this.options.startpageId;
			}
			else{
				returnData.ObjectID = node.data('jstree').id;
			}
			return returnData;
		},

		_getAdvancedLinks: function(){
			var self = this,
				o = self.options;

			ep.ajax({
					type:		'post',
					dataType:	'json',
					data:		{
						ViewAction:			'JSONAdvancedLinks',
						ObjectID:			ep.config.siteId
					}
				})
				.done(function( data ){
					var sfUrl = ep.config.storeFrontUrl;

					$(data).each(function(i,elem){
						var linkUrl = elem.Link;

						if(o.useAbsolute){
							var testRegEx = /^(.*)\?/g;
							// weburl contains query string?
							if(testRegEx.test(sfUrl) == true) {
								testRegEx.exec(sfUrl);
								linkUrl = RegExp.$1 + linkUrl;
							} else {
								linkUrl = sfUrl + linkUrl;
							}
						}

						$('<option>')
							.val(linkUrl)
							.html(elem.Alias)
							.appendTo(self.additionalSelect);
					});

					self.additionalSelect.on('change',$.proxy(self,'_onSelectChange'));
				});
		},

		_onSelectChange:function(elem){
			var self = this,
				selectedOption = $(elem.currentTarget).children(':selected');

			self.urlInput.val(selectedOption.val());

			if(!self.titleInput.attr('disabled')){
				self.titleInput.val(selectedOption.text());
			}
		},

		destroy: function(){
			if(this.dialog){
				this.dialog.remove();
			}

			this.Inherited(arguments);
		}

	});

});
