use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::UPS::API::Soap::TransformErrorMessages;
use Test::More;
use Data::Dumper;
sub Main {
    testErrorHandling();
    #test_addLinkToErrorRange(); # hier muss ich jetzt noch das ganze Shippinglabel nachzeiehn
    done_testing();
}
sub test_addLinkToErrorRange {
    my $TransformErrorMessage = DE_EPAGES::UPS::API::Soap::TransformErrorMessages->new();
    ## no critic (ProhibitPrivatesubroutineUse ProhibitMagicNumbers RequireNumberSeparators)
    my $GroupExpand = $TransformErrorMessage->_linkToUPSAccountSetup();
    is(TestErrorGroups(120119),undef,'SomeUPSError Test(no expand): 120119');
    is(TestErrorGroups(120120),$GroupExpand,'UPSAccountError Test: 120120');
    is(TestErrorGroups(120122),$GroupExpand,'UPSAccountError Test: 120122');
    is(TestErrorGroups(120123),undef,'SomeUPSError Test(no expand): 120123');

    $GroupExpand = $TransformErrorMessage->_linkToUPSOrderSetup();
    is(TestErrorGroups(120199),undef,'SomeUPSError Test(no expand): 120199');
    is(TestErrorGroups(120200),$GroupExpand,'UPSOrderError Test: 120200');
    is(TestErrorGroups(120218),$GroupExpand,'UPSOrderError Test: 120218');
    is(TestErrorGroups(120219),undef,'SomeUPSError Test(no expand): 120219');

    $GroupExpand = $TransformErrorMessage->_linkToShopAddressSetup();
    is(TestErrorGroups(120299),undef,'SomeUPSError Test(no expand): 120299');
    is(TestErrorGroups(120300),$GroupExpand,'UPSShopAddress Test: 120300');
    is(TestErrorGroups(120318),$GroupExpand,'UPSShopAddress Test: 120318');
    is(TestErrorGroups(120319),undef,'SomeUPSError Test(no expand): 120319');
}

sub TestErrorGroups {
    my ( $Code ) = @_;
    my $Error;
    my $TransformErrorMessage = DE_EPAGES::UPS::API::Soap::TransformErrorMessages->new();
    eval {
             Error( 'Test', {
                'Code' => $Code,
                'Description' => 'Test_Description',
        } );
    };
    if (ExistsError()) {
        $Error = GetError();
        $Error = $TransformErrorMessage->_addLinkToErrorRange($Error);## no critic (ProhibitPrivatesubroutineUse)
        my $hErrorVars = $Error->vars;
        is ($hErrorVars->{'Code'}, $Code, "test group error code: $Code");
        is ($hErrorVars->{'Description'}, 'Test_Description', "test group error code: $Code");
    }
    return $Error->{'Vars'}->{'Link'};
}

sub testErrorHandling {
    eval {
        Error( 'Wurst', {
                'Code' => '120121',
                'Description' => 'Wurst_Description',
        } );
    };
    if (ExistsError()) {
        my $TransformErrorMessage = DE_EPAGES::UPS::API::Soap::TransformErrorMessages->new();
        my $Error = GetError();
        $Error = $TransformErrorMessage->doTransform($Error);
        # tests
        is ($Error->code, 'Wurst', 'test error name');
        my $hErrorVars = $Error->vars;
        is ($hErrorVars->{'Code'},'120121', 'test error code');
        is ($hErrorVars->{'Description'},'Wurst_Description', 'test error description attachment');
    }
    eval{
        Error( 'DoofDoof', {
                'Code' => '110',
        } );
    };
    if (ExistsError()) {
        my $TransformErrorMessage = DE_EPAGES::UPS::API::Soap::TransformErrorMessages->new();
        my $Error = GetError();
        $Error = $TransformErrorMessage->doTransform($Error);
        # tests
       is ($Error->code, 'DoofDoof', 'not matched test error name');
        my $hErrorVars = $Error->vars;
       is ($hErrorVars->{'Code'},'110', 'not matched test error code');
       is ($hErrorVars->{'Description'},undef, 'not matched test error description attachment');
    }
}

RunScript( Sub => \&Main );